/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package cin.aquario;

import java.awt.*;
import java.util.*;

import javax.media.j3d.*;
import javax.swing.*;
import javax.vecmath.*;

import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import org.simulare.*;

/**
 * A 3D report to represent aquarium items. The aquarium is represented by a
 * transparent box, the coral is a green sphere, the ordinary fish is a yellow
 * sphere and the predator is a red sphere.
 */
public class Report3D extends AbstractTask implements VisualTask {

    // visualization JPanel;
    private JPanel panel;
    // Java3D elements
    private SimpleUniverse universe;
    private BranchGroup root;
    private TransformGroup transGroup;
    // mapping from element to 3D object
    private Map elements;
    // box dimension
    private Attribute altura, largura, comprimento;

    public Report3D() {
       this("report3d");
    }

    public Report3D(String name) {
        super(name);
        panel = new JPanel();
        panel.setLayout(new BorderLayout());
        //create universe
        universe = UniverseBuilder.getUniverse();
        Canvas3D canvas = universe.getCanvas();
        canvas.setBackground(Color.LIGHT_GRAY);
        //add universe canvas to the report panel
        panel.add(canvas, BorderLayout.CENTER);
        elements = new HashMap();

        // the base of universe itens
        root = new BranchGroup();
        root.setCapability(Group.ALLOW_CHILDREN_EXTEND);
        root.setCapability(Group.ALLOW_CHILDREN_READ);
        root.setCapability(Group.ALLOW_CHILDREN_WRITE);
        // the base tranform group
        transGroup = new TransformGroup();
        transGroup.setCapability(Group.ALLOW_CHILDREN_EXTEND);
        transGroup.setCapability(Group.ALLOW_CHILDREN_READ);
        transGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);
        transGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        transGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        // execute a zoom out on transform group
        Transform3D trans = new Transform3D();
        Vector3d point = new Vector3d(0, 0, -10);
        trans.setTranslation(point);
        transGroup.setTransform(trans);
        // the limits that will be controled by mouse, and where lights
        // can act
        BoundingSphere bounds = new BoundingSphere(new Point3d(0, 0, 0), 300);
        // the mouse actions
        MouseBehavior behavior = new MouseRotate();
        behavior.setTransformGroup(transGroup);
        transGroup.addChild(behavior);
        behavior.setSchedulingBounds(bounds);
        behavior = new MouseTranslate();
        behavior.setTransformGroup(transGroup);
        transGroup.addChild(behavior);
        behavior.setSchedulingBounds(bounds);
        behavior = new MouseZoom();
        behavior.setTransformGroup(transGroup);
        transGroup.addChild(behavior);
        behavior.setSchedulingBounds(bounds);
        // lights acting on the aquario
        BranchGroup lights = new BranchGroup();
        lights.setCapability(Group.ALLOW_CHILDREN_EXTEND);
        lights.setCapability(Group.ALLOW_CHILDREN_READ);
        lights.setCapability(Group.ALLOW_CHILDREN_WRITE);
        // shine it with two colored lights.
        // a white
        Color3f lColor1 = new Color3f(1.0f, 1.0f, 1.0f);
        // and blue light
        Color3f lColor2 = new Color3f(0.0f, 0.0f, 1.0f);
        Vector3f lDir1 = new Vector3f( -1.0f, -1.0f, -1.0f);
        Vector3f lDir2 = new Vector3f(0.0f, 0.0f, -1.0f);
        // two directional lights
        DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
        DirectionalLight lgt2 = new DirectionalLight(lColor2, lDir2);
        // one pointing light with color yellow.
        PointLight lgt3 = new PointLight(true, new Color3f(Color.YELLOW),
                                         new Point3f( -1, 1, 1),
                                         new Point3f(1, 0, 0));
        // set the light influence regions
        lgt1.setInfluencingBounds(bounds);
        lgt2.setInfluencingBounds(bounds);
        lgt3.setInfluencingBounds(bounds);
        lights.addChild(lgt1);
        lights.addChild(lgt2);
        lights.addChild(lgt3);
        // add lights to the universe
        root.addChild(lights);
        // add the base transform group to the environment
        root.addChild(transGroup);
        // add root element to the universe
        universe.addBranchGraph(root);
    }

    public void execute(Item item) {
        ItemGroup ig = (ItemGroup) elements.get(item.getName());
        TransformGroup tg = null;
        if( ig != null ) {
            tg = ig.tg;
        }
        // if the item is an aquarium
        if (item.getName().startsWith("aquario")) {
            if (tg == null) {
                // get box attributes
                altura = item.getAttribute(ActionMove.PRODUCERX);
                largura = item.getAttribute(ActionMove.PRODUCERY);
                comprimento = item.getAttribute(ActionMove.PRODUCERZ);
                // create a transparent box
                BranchGroup group = new BranchGroup();
                group.setCapability(BranchGroup.ALLOW_DETACH);
                group.setCapability(Group.ALLOW_CHILDREN_EXTEND);
                group.setCapability(Group.ALLOW_CHILDREN_READ);
                group.setCapability(Group.ALLOW_CHILDREN_WRITE);
                tg = new TransformGroup();
                tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
                tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                // the box appearance
                Appearance ap = new Appearance();
                Color3f objColor = new Color3f(Color.WHITE);
                Color3f black = new Color3f(Color.BLACK);
                Color3f white = new Color3f(Color.WHITE);
                // set material, and color, to allow light reflection
                ap.setMaterial(new Material(objColor, black, objColor, white,
                                            80.0f));
                ap.setColoringAttributes(new ColoringAttributes(objColor,
                        ColoringAttributes.FASTEST));
                //70% transparent to allow seeing aquario items
                ap.setTransparencyAttributes(new TransparencyAttributes(
                        TransparencyAttributes.FASTEST, 0.70f));
                // create the box with the given appearance
                com.sun.j3d.utils.geometry.Box aquario = new com.sun.j3d.utils.
                        geometry.Box(altura.getFloat(), largura.getFloat(),
                                     comprimento.getFloat(), ap);
                // add the aquario to its transform group
                tg.addChild(aquario);
                // add transform to a root element
                group.addChild(tg);
                // add group to the main transform group
                transGroup.addChild(group);
                // put the element into elements mapping
                ig = new ItemGroup();
                ig.bg = group;
                ig.tg = tg;
                elements.put(item.getName(), ig);
            }
        } else {
            Attribute alive = item.getAttribute(LivingThing.IS_ALIVE);
            //JOptionPane.showMessageDialog(panel,item.getName()+" has dead","Something has dead",JOptionPane.OK_OPTION);
            if( alive.getBoolean() ) {
                // get item position attributes
                Attribute x = item.getAttribute(ActionMove.CONSUMERX);
                Attribute y = item.getAttribute(ActionMove.CONSUMERY);
                Attribute z = item.getAttribute(ActionMove.CONSUMERZ);
                // the placement of the item represents a transformation into model
                Transform3D trans = new Transform3D();
                //ninja factor
                Vector3d point = new Vector3d(x.getFloat(), y.getFloat(),
                                              z.getFloat());
                trans.setTranslation(point);
                // if there is no transform group the the item create one
                if (tg == null) {
                    BranchGroup group = new BranchGroup();
                    group.setCapability(BranchGroup.ALLOW_DETACH);
                    group.setCapability(Group.ALLOW_CHILDREN_EXTEND);
                    group.setCapability(Group.ALLOW_CHILDREN_READ);
                    group.setCapability(Group.ALLOW_CHILDREN_WRITE);
                    tg = new TransformGroup();
                    tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
                    tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                    Appearance ap = new Appearance();
                    Color3f objColor = new Color3f(1, 1, 0);
                    //depending on item type change the color
                    if (item.getName().startsWith("coral")) {
                        objColor = new Color3f(Color.green);
                    } else if (item.getName().startsWith("presa")) {
                        objColor = new Color3f(Color.yellow);
                    } else if (item.getName().startsWith("predador")) {
                        objColor = new Color3f(Color.red);
                    }
                    Color3f black = new Color3f(Color.BLACK);
                    Color3f white = new Color3f(Color.WHITE);
                    ap.setMaterial(new Material(objColor, black, objColor,
                                                white,
                                                80.0f));
                    // these items are represented by spheres
                    Sphere s = new Sphere(0.4f, Sphere.GENERATE_NORMALS, 50, ap);
                    //add sphere to the item transform group
                    tg.addChild(s);
                    // add transform to a group
                    group.addChild(tg);
                    // add the group to the main transform group
                    transGroup.addChild(group);
                    // put the element into elements mapping
                    ig = new ItemGroup();
                    ig.bg = group;
                    ig.tg = tg;
                    elements.put(item.getName(), ig);
                }
                // perform the transform for the item (change its position)
                tg.setTransform(trans);
            } else {
                if( elements.get(item.getName()) != null ) {
                    JOptionPane.showMessageDialog(panel,
                                                  item.getName() + " was eaten.\n"+
                                                  "It will be removed from the model.",
                                                  "Death notifier",
                                                  JOptionPane.INFORMATION_MESSAGE);
                    elements.remove(item.getName());
                    transGroup.removeChild(ig.bg);
                }
            }
        }
    }

    class ItemGroup {
        public BranchGroup bg;
        public TransformGroup tg;
    }

    public Component getEditor() {
        return null;
    }

    public Component getView() {
        return panel;
    }
}