/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

/**
 * Define a clock manager responsible for simulating a time schedule for executing
 * some operations. This clock fires events in a periodic rate defined by the
 * attribute period. To allow a better simulation of parallelism the fireEvent
 * method notifys listeners using a random order.
 */
public class Clock {
    /** The scheduler. */
    private Timer timer;

    /** Indicate if the clock is on pause state. */
    private boolean paused;

    /** The period to fire clock event. */
    private long period;

    /** The max number of ticks. */
    private long maxCounter;

    /** The listeners list. */
    private transient Vector clockListeners;
    private static final int START = 0;
    private static final int PAUSE = START + 1;
    private static final int RESUME = PAUSE + 1;
    private static final int STOP = RESUME + 1;
    private static final int UPDATE = STOP + 1;
    /** The random generator */
    private Random random;
    /** A shared event to avoid objet creation. */
    private ClockEvent event;

    /**
     * Creates a clock with a period of 1 second, and maxCounter to positive
     * infinite.
     */
    public Clock() {
        this(500, Long.MAX_VALUE);
    }

    /**
     * Creates a clock with a period.
     * @param period The periodic Thread interval execution.
     * @param maxCounter The max number of clock ticks.
     */
    public Clock(long period, long maxCounter) {
        setPeriod(period);
        setMaxCounter(maxCounter);
        paused = false;
        random = new Random();
        event = new ClockEvent(this);
        event.counter = 0;
        event.stoped = true;
    }

    /**
     * The clock period in miliseconds.
     * @return The clock period.
     */
    public long getPeriod() {
        return period;
    }

    /**
     * Set the clock period. The period only has efect if the clock has not
     * started yet, or the clock is in pause, is this case the clock can resume
     * its execution with a new period.
     * @param period The clock period.
     */
    public void setPeriod(long period) {
        this.period = period;
    }

    /**
     * Indicate if the clock is in pause.
     * @return true, the clock is paused, false, otherwise.
     */
    public boolean isPaused() {
        return paused;
    }

    /**
     * Return the max number of clock ticks.
     * @return The max number of clock ticks.
     */
    public long getMaxCounter() {
        return maxCounter;
    }

    /**
     * Set the max counter for the clock.
             * @param maxCounter The max cicle counter. After this limit has been reached
     * the clock stops.
     */
    public void setMaxCounter(long maxCounter) {
        this.maxCounter = maxCounter;
    }

    /**
     * Start the clock. After started the clock start to fire events.
     */
    public void start() {
        init();
        //after init call clockStarted.
        fireClockStarted(event);
    }

    /**
     * Inits the timer, on start or on resume.
     */
    private void init() {
        // create the timer and schedule the clock.
        timer = new Timer();
        // schedule ata fixed period of times
        TimerTask task = new TimerTask() {
            public void run() {
                if (!paused) {
                    if (event.counter > maxCounter) {
                        // if the counter has exceeded, decrement and call
                        // stop().
                        event.counter--;
                        stop();
                    } else {
                        // fire event before increment counter
                        fireClockChanged(event);
                        event.counter++;
                    }
                }
            }
        };
        timer.scheduleAtFixedRate(task, 0, period);
        //return the clock to zero, otherwise, continue counting
        if (!paused) {
            event.counter = 0;
        }
        event.stoped = false;
    }

    /**
     * Interrupts the clock counting.
     */
    public void pause() {
        // finish current schedule
        timer.cancel();
        // put in pause state
        paused = true;
        fireClockPaused(event);
    }

    /**
     * Resume the clock execution.
     */
    public void resume() {
        // a new timer is required because the period can be changed while is
        // pause state.
        init();
        // make thread available
        paused = false;
        //after init call clockResumed.
        fireClockResumed(event);
    }

    /**
     * Stop the clock.
     */
    public void stop() {
        //finish all thread
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
        paused = false;
        event.stoped = true;
        fireClockStoped(event);
    }

    /**
     * Add a new clock listener.
     * @param l The clock listener.
     */
    public synchronized void removeClockListener(ClockListener l) {
        if (clockListeners != null && clockListeners.contains(l)) {
            Vector v = (Vector) clockListeners.clone();
            v.removeElement(l);
            clockListeners = v;
        }
    }

    /**
             * Remove a clock listener added. If the listener was not added before, this
     * method has no efect.
     * @param l The clock listener.
     */
    public synchronized void addClockListener(ClockListener l) {
        Vector v = clockListeners == null ? new Vector(2) :
                   (Vector) clockListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            clockListeners = v;
        }
    }

    protected void fireClockStarted(ClockEvent e) {
        randomSelect(e, START);
    }

    protected void fireClockPaused(ClockEvent e) {
        randomSelect(e, PAUSE);
    }

    protected void fireClockResumed(ClockEvent e) {
        randomSelect(e, RESUME);
    }

    protected void fireClockStoped(ClockEvent e) {
        randomSelect(e, STOP);
    }

    protected void fireClockChanged(ClockEvent e) {
        randomSelect(e, UPDATE);
    }

    /**
             * Fire an event to listeners. This method rely on java.util.Random.nextInt()
     * to try calling different listeners.
     * @param e The clock event.
     * @param type The event type.
     */
    private synchronized void randomSelect(ClockEvent e, int type) {
        if (clockListeners != null) {
            Vector listeners = (Vector) clockListeners.clone();
            int count = listeners.size();
            String str = null;
            switch (type) {
            case START:
                str = "start";
                break;
            case PAUSE:
                str = "pause";
                break;
            case RESUME:
                str = "resume";
                break;
            case STOP:
                str = "stop";
                break;
            case UPDATE:
                str = "update";
                break;
            }

            System.out.println(
                    "+--------------------- START " + str + " (" +
                    event.counter +
                    ") -----------------------+");

            for (int i = 0; i < count; i++) {
                int next = Math.abs(random.nextInt()) % (count - i);
                //System.out.println("NEXT(" + i + "."+ listeners.size() + "):" + next);
                switch (type) {
                case START:
                    ((ClockListener) listeners.elementAt(next)).clockStarted(e);
                    break;
                case PAUSE:
                    ((ClockListener) listeners.elementAt(next)).clockPaused(e);
                    break;
                case RESUME:
                    ((ClockListener) listeners.elementAt(next)).clockResumed(e);
                    break;
                case STOP:
                    ((ClockListener) listeners.elementAt(next)).clockStoped(e);
                    break;
                case UPDATE:
                    ((ClockListener) listeners.elementAt(next)).clockChanged(e);
                    break;
                }
                listeners.remove(next);
            }
            System.out.println(
                    "+--------------------- END " + str + " (" + event.counter +
                    ") -----------------------+");
        }
    }
}