/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

/**
 * Represents a simulation item.
 */
public class Item extends Named implements AttributeListener {
    private CompositeItem parent;
    private List attributes_order;
    private Map attributes;
    private transient Vector itemListeners;
    protected ItemEvent event;

    public Item(String name) {
        super(name);
        attributes_order = new LinkedList();
        attributes = new HashMap();
        event = new ItemEvent(this);
    }

    public CompositeItem getParent() {
        return parent;
    }

    public void setParent(CompositeItem parent) {
        this.parent = parent;
    }

    public void addAttribute(Attribute attribute) {
        addAttribute(attributes_order.size(), attribute);
    }

    public void addAttribute(int index, Attribute attribute) {
        String name = attribute.getName();
        if (!attributes_order.contains(name)) {
            attributes_order.add(index, name);
        }
        attributes.put(name, attribute);
        attribute.addAttributeListener(this);
        fireItemStateChanged(event);
    }

    public Attribute removeAttribute(String name) {
        attributes_order.remove(name);
        Attribute result = (Attribute) attributes.remove(name);
        fireItemStateChanged(event);
        return result;
    }

    public Attribute removeAttribute(int index) {
        return removeAttribute((String) attributes_order.get(index));
    }

    public Attribute getAttribute(String name) {
        return (Attribute) attributes.get(name);
    }

    public Attribute getAttribute(int index) {
        return getAttribute((String) attributes_order.get(index));
    }

    public int getAttributeIndex(String name) {
        return attributes_order.indexOf(name);
    }

    public String getAttributeName(int index) {
        return (String) attributes_order.get(index);
    }

    public Iterator getAttributesNames() {
        return attributes_order.iterator();
    }

    public int getAttributesLength() {
        return attributes_order.size();
    }

    public Item toItem() {
        return copyAttributes(new Item(getName()));
    }

    public CollectionItem toCollection() {
        return (CollectionItem) copyAttributes(new CollectionItem(getName()));
    }

    public CompositeItem toComposite() {
        return (CompositeItem) copyAttributes(new CompositeItem(getName()));
    }

    protected Item copyAttributes(Item result) {
        for (int i = 0; i < getAttributesLength(); i++) {
            result.addAttribute(getAttribute(i).getCopy());
        }
        return result;
    }

    public void typeChanged(AttributeEvent e) {
        fireAttribute(e);
    }

    public void valueChanged(AttributeEvent e) {
        fireAttribute(e);
    }

    private void fireAttribute(AttributeEvent e) {
        ItemEvent ev = new ItemEvent(this);
        ev.attribute = e;
        fireItemStateChanged(ev);
    }

    public synchronized void removeItemListener(ItemListener l) {
        if (itemListeners != null && itemListeners.contains(l)) {
            Vector v = (Vector) itemListeners.clone();
            v.removeElement(l);
            itemListeners = v;
        }
    }

    public synchronized void addItemListener(ItemListener l) {
        Vector v = itemListeners == null ? new Vector(2) :
                   (Vector) itemListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            itemListeners = v;
        }
    }

    protected void fireItemStateChanged(ItemEvent e) {
        if (itemListeners != null) {
            Vector listeners = itemListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((ItemListener) listeners.elementAt(i)).itemStateChanged(e);
            }
        }
    }
}