/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare.gui;

import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;

import org.simulare.*;

/**
 * A smarter tree with event to identify singleClick, doubleClick or popupClick
 * events.
 */
public class JTreeSmart extends JTree {

    private transient Vector JTreeSmartListeners;

    public JTreeSmart(TreeNode root) {
        this(new DefaultTreeModel(root));
    }

    public JTreeSmart(TreeModel model) {
        super(model);
        addMouseListener(new ClickListener());
    }

    private class ClickListener extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            mayShowPopup(e);
        }

        public void mouseReleased(MouseEvent e) {
            mayShowPopup(e);
        }

        private void mayShowPopup(MouseEvent e) {
            int selRow = getRowForLocation(e.getX(), e.getY());
            TreePath selPath = getPathForLocation(e.getX(), e.getY());
            if (e.isPopupTrigger()) {
                firePopupClick(new JTreeSmartEvent(e, selRow, selPath));
            }
        }

        public void mouseClicked(MouseEvent e) {
            int selRow = getRowForLocation(e.getX(), e.getY());
            TreePath selPath = getPathForLocation(e.getX(), e.getY());
            if (selRow != -1) {
                if (e.getClickCount() == 1) {
                    fireSingleClick(new JTreeSmartEvent(e, selRow, selPath));
                } else if (e.getClickCount() == 2) {
                    fireDoubleClick(new JTreeSmartEvent(e, selRow, selPath));
                }
            }
        }
    }


    public void expandCurrent() {
        TreePath path = getSelectionPath();
        if (path != null) {
            command(path, true);
        }
    }

    public void expandAll() {
        CompositeItem root = (CompositeItem) getModel().getRoot();
        command(new TreePath(root), true);
    }

    public void colapseCurrent() {
        TreePath path = getSelectionPath();
        if (path != null) {
            command(path, false);
        }
    }

    public void colapseAll() {
        CompositeItem root = (CompositeItem) getModel().getRoot();
        command(new TreePath(root), false);
    }

    private void command(TreePath parent, boolean expand) {
        Item node = (Item) parent.getLastPathComponent();
        if (node instanceof CompositeItem) {
            CompositeItem ci = (CompositeItem) node;
            if (ci.getItemsLength() >= 0) {
                for (int i = 0; i < ci.getItemsLength(); i++) {
                    TreePath path = parent.pathByAddingChild(ci.getItem(i));
                    command(path, expand);
                }
            }
        }
        if (expand) {
            expandPath(parent);
        } else {
            collapsePath(parent);
        }
    }

    public synchronized void removeJTreeSmartListener(JTreeSmartListener l) {
        if (JTreeSmartListeners != null && JTreeSmartListeners.contains(l)) {
            Vector v = (Vector) JTreeSmartListeners.clone();
            v.removeElement(l);
            JTreeSmartListeners = v;
        }
    }

    public synchronized void addJTreeSmartListener(JTreeSmartListener l) {
        Vector v = JTreeSmartListeners == null ? new Vector(2)
                   : (Vector) JTreeSmartListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            JTreeSmartListeners = v;
        }
    }

    protected void fireSingleClick(JTreeSmartEvent e) {
        if (JTreeSmartListeners != null) {
            Vector listeners = JTreeSmartListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((JTreeSmartListener) listeners.elementAt(i)).singleClick(e);
            }
        }
    }

    protected void fireDoubleClick(JTreeSmartEvent e) {
        if (JTreeSmartListeners != null) {
            Vector listeners = JTreeSmartListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((JTreeSmartListener) listeners.elementAt(i)).doubleClick(e);
            }
        }
    }

    protected void firePopupClick(JTreeSmartEvent e) {
        if (JTreeSmartListeners != null) {
            Vector listeners = JTreeSmartListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((JTreeSmartListener) listeners.elementAt(i)).popupClick(e);
            }
        }
    }
}
